# CovrGC
# Computing Generalized Covariance and Correlation along a Principal Curve
# CovGC CorGC
# using the "principal.curve" function (Hastie-Stuetzle Principal Curve)
# from library "princurve"
# or using the "pcop" function (Principal Curve of Oriented Points),
# depending on the argument "method".
#
# Arguments:
#         x: a matrix of n points in dimension 2
#    method: "HS" for using Hastie-Stuetzle principal curve, 
#            "PCOP" for using PCOP
#  h.method: method to choose the smoothing parameter in the internal local regression calls
# plot.true: if TRUE, the function produce a plot
#       ...: Additional parameters passed to principal curve functions
# 
# Value: A list with elements
#      PC: Pricipal curve fitted to points in x
#   CovGC: Generalized Covariance along a Principal Curve
#   CorGC: Generalized Correlation along a Principal Curve
#  LCovGC: Local Generalized Covariance along a Principal Curve
#  LCorGC: Local Generalized Correlation along a Principal Curve
#   
# Require: princurve, KernSmooth, functions "pcop", "localpolreg_no_plot"
Covr <- function(x, method="HS", h.method="dpill", plot.true=FALSE, ...){
 
   require(princurve)
   require(KernSmooth)

# Computing the Principal Curve
    PC <- switch(method,
             hs=, HS = principal.curve(x,plot.true=plot.true,...),
             pcop=, PCOP = pcop(x,plot.true=plot.true,...)$pcop.f2
          )

# Calculating plug-in bandwidths   
##   h1 <- dpill(PC$lambda,PC$s[,1]) # dpill does not work for x and y
##   h2 <- dpill(PC$lambda,PC$s[,2]) # arrays equally ordered


# Estimating derivatives of the PC
#   "locpoly(...)" does not admit a xgrid array of points
#   where evaluating the regression function.
#   I use my "locpolreg(...)" function instead.
   h1 <- diff(range(PC$lambda))/10
   h2 <- 2*max(diff(sort(PC$lambda)))
   hlambda <- max(h1,h2) 
   lpr1 <- locpolreg_no_plot(PC$lambda,PC$s[,1],p=2,r=1,h=hlambda)
   lpr2 <- locpolreg_no_plot(PC$lambda,PC$s[,2],p=2,r=1,h=hlambda)
   # (Note: lpr*%x and lpr*$y are sorted according to lpr*$x)


# Computing angles alpha(s)
   alpha <- atan2(lpr1$mxgr, lpr2$mxgr)
   aux <- sort(PC$tag, ind=T)
   # recovering original ordering
#points(PC$lambda, lpr2$mxgr[aux$ix],col=2)

   alpha <- alpha[aux$ix]
 
# Estimating conditional orthogonal variance.
#  e2 <- apply((x - PC$s)^2,1,sum)
   nn <- dim(x)[1]
   e2 <- array(apply((x - PC$s)^2,1,sum),dim=c(nn,1))[,1]
   if (h.method != "dpill"){
      he2_1 <- diff(range(PC$lambda))/5
   }else{
      he2 <- dpill(PC$lambda, e2,trim=.01)
      cte.h <- 2.214 # h1 and h2 are computed for Gaussian kernel, and
                     # locpolreg uses Epanechnikov kernel
      he2_1 <- he2*cte.h
      # prevention against NaN produced by dpill
      if (is.na(he2_1)) he2_1 <- diff(range(PC$lambda))/5
   }   
   he2_2 <- 2*max(diff(sort(PC$lambda)))
   he2 <- max(he2_1,he2_2) 
   CondOrthVar <- locpolreg_no_plot(PC$lambda, e2,h=he2)$mxgr
   # recovering original ordering and preventing for negative estimations
   CondOrthVar <- pmax(CondOrthVar[aux$ix],0) 

# Var. over the PC
   VS <- var(PC$lambda)

# Computing local varinaces, covariance and correlation
   LV1 <- VS * cos(alpha)^2 + CondOrthVar*sin(alpha)^2
   LV2 <- VS * sin(alpha)^2 + CondOrthVar*cos(alpha)^2
   LCovGC <- (VS - CondOrthVar)*sin(alpha)*cos(alpha)
   LCorGC <- LCovGC/sqrt(LV1*LV2)

# Computing Generalized Covariance and Correlation along GC
   CovGC <- sqrt(mean(LCovGC^2))
   CorGC <- sqrt(mean(LCorGC^2))

# Value:
   return(list(PC=PC,CovGC=CovGC,CorGC=CorGC,LCovGC=LCovGC,LCorGC=LCorGC))
}
